﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Windows;

namespace ListValidation
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();

            var people = new HasParentObservableCollection<Person>();
            people.Add(new Person(){ FirstName = "Brian", LastName = "Lagunas"});
            xamDataGrid1.DataSource = people;
        }

        public class HasParentObservableCollection<T> : ObservableCollection<T>
        {
            protected override void InsertItem(int index, T item)
            {
                //set the parent object when a new item is added to our collection
                if (item != null && item is IHasParent)
                    (item as IHasParent).Parent = this;

                base.InsertItem(index, item);
            }
        }

        public interface IHasParent
        {
            object Parent { get; set; }
        }

        public class Person : INotifyPropertyChanged, IDataErrorInfo, IHasParent
        {
            public object Parent { get; set; }

            private string _firstName;
            public string FirstName
            {
                get { return _firstName; }
                set
                {
                    _firstName = value;
                    OnPropertyChanged("FirstName");
                }
            }

            private string _lastName;
            public string LastName
            {
                get { return _lastName; }
                set
                {
                    _lastName = value;
                    OnPropertyChanged("LastName");
                }
            }

            public event PropertyChangedEventHandler PropertyChanged;
            protected void OnPropertyChanged(string propertyName)
            {
                var handler = PropertyChanged;
                if (handler != null)
                    handler(this, new PropertyChangedEventArgs(propertyName));
            }

            public string this[string columnName]
            {
                get
                {
                    if (columnName == "FirstName")
                    {
                        if (String.IsNullOrWhiteSpace(FirstName))
                            return "First name cannot be empty";
                    }

                    if (columnName == "LastName")
                    {
                        if (String.IsNullOrWhiteSpace(LastName))
                            return "Last name cannot be empty";

                        if (Parent != null && Parent is IList<Person>)
                        {
                            var list = (IList<Person>) Parent;
                            if (list.Count(x => x.LastName == LastName) > 1)
                                return "This last name already exists.  Please use a different last name.";
                        }
                    }

                    return String.Empty;
                }
            }

            public string Error
            {
                get { return String.Empty; }
            }
        }
    }
}
